/*
 * VID2IM
 * -------------------------------------------
 * MIGUEL TORRES TORRITI (c) 2007 v. 1.0
 * -------------------------------------------
 *
 * To run type:
 *  vid2im     -> to convert a video captured with the first webcam attached to the PC.
 *                into a sequence of images.
 *  vid2im [n] -> to convert a video captured with the n-th webcam attached to the PC.
 *                into a sequence of images.
 *  vid2im [input video filename=video.avi] [options] [output image basefilename=f]
 *             -> to convert a video file into a sequence of images.
 *
 * Description: 
 *  Converts a video to a sequence of images.  By default this function generates
 *  files with base name "f" and indeces in the range from 0 to 10 with three zero-padding,
 *  i.e. f000.jpg, f001.jpg, ..., f010.jpg, at a frame rate that depends on the hardware.
 *
 * Options:
 *  -h   Prints this help.
 *  -nv  Turns verbose off.
 *  -np  Do not wait for user to press ESC when done.
 *  -nd  Do not display images.
 *  -no  Do not save the output.
 *  -e   Filename extension [=jpg]
 *  -sq initial_value final_value  Tells vid2im to generate a sequence of images
 *                                 starting at filename<initial_value>.ext and 
 *                                 ending in filename<initial_value>.ext
 *            By default the sequence range is assumed to be between 0 and 10 
 *            with zero-padding = 3, i.e. 000, 001, ..., 010.
 *  -zp num   Pad filename suffix with num zeros.
 *  -fps num  Sets the frame acquisition rate (in frames per second).
 *            This option is ignored in the current implementation.
 *
 * Author: Miguel Torres Torriti (c) 2007.12.15, v. 1.
 *
 */
#include "cv.h"       // use " " instead of < > to get list of class/struct members
#include "highgui.h"  // use " " instead of < > to get list of class/struct members
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>

struct cmdlinestruct {
	char basefilename[256];		// Frame base filename
	char fileextension[250];	// Frame file extension
	char filenameformat[200];	// "%s%0<n>d.%s" (filename)(zero-padding number)(extension)
	int iv;						// Frame initial value
	int fv;						// Frame final value
	double fps;					// Video writer parameters
	char filename_in[512];		// Input video filename
	bool output_flag;
	bool display_flag;
	bool verbose_flag;
	bool pause_flag;
};

int initcmdlinestruct(cmdlinestruct* cmdlineparam)
{
	sprintf((*cmdlineparam).basefilename,"%s","f");			// Frame base filename
	sprintf((*cmdlineparam).fileextension,"%s","jpg");		// Frame file extension
	sprintf((*cmdlineparam).filenameformat,"%%s%%0%dd.%%s",3);	// Builds file suffix format string "%s%0<n>d.%s"
																// where <n> is the number for zero padding.
	(*cmdlineparam).iv=0;									// Frame initial value
	(*cmdlineparam).fv=10;									// Frame final value
	(*cmdlineparam).fps=15.0;								// Video writer parameters
	sprintf((*cmdlineparam).filename_in,"%s","video.avi");	// Output video filename
	(*cmdlineparam).output_flag=1;
	(*cmdlineparam).display_flag=1;
	(*cmdlineparam).verbose_flag=1;
	(*cmdlineparam).pause_flag=1;
	return 0;
}

int parsecmdline(int argc, char** argv, cmdlinestruct* cmdlineparam)
{
	// Parse command line options
	for (int i=1; i<argc; i++)
	{	
		if (strcspn( argv[i], "-" )==0 && strstr(argv[i],"-nv")!=NULL)
		{
			(*cmdlineparam).verbose_flag = 0;
			break;
		}
	}
	if ((*cmdlineparam).verbose_flag) printf("Verbose is on.\n");

	for (int i=1; i<argc; i++)
	{	
		if (strcspn( argv[i], "-" )==0)
		{
			if (strstr(argv[i],"-nv")!=NULL)
			{
				(*cmdlineparam).verbose_flag = 0;
			}
			else if (strstr(argv[i],"-np")!=NULL)
			{
				(*cmdlineparam).pause_flag = 0;
			}
			else if (strstr(argv[i],"-nd")!=NULL)
			{
				if ((*cmdlineparam).verbose_flag) printf("No display.\n");
				(*cmdlineparam).display_flag = 0;
			}
			else if (strstr(argv[i],"-no")!=NULL)
			{
				if ((*cmdlineparam).verbose_flag) printf("No output.\n");
				(*cmdlineparam).output_flag = 0;
			}
			else if (strstr(argv[i],"-h")!=NULL)
			{
				printf(
					"vid2im     -> to convert a video captured with the first webcam attached to the\n"
					"              PC into a sequence of images.\n"
					"vid2im [n] -> to convert a video captured with the n-th webcam attached to the\n"
					"              PC into a sequence of images.\n"
					"vid2im [input video filename=video.avi] [options] [output image basefilename=f]\n"
					"           -> to convert a video file into a sequence of images.\n"
					"\n"
					"Description:\n"
					"\tConverts a video to a sequence of images.  By default this function\n"
					"\tgenerates files with base name \"f\" and indices in the range\n"
					"\tbetween 0 to 10 with three zero padding, i.e. f000.jpg, f001.jpg, ...,\n"
					"\tf010.jpg, at a frame rate that depends on the hardware.\n"
					"\n"
					"Options:\n"
					"\t-h   Prints this help.\n"
					"\t-nv  Turns verbose off.\n"
					"\t-np  Do not wait for user to press ESC when done.\n"
					"\t-nd  Do not display images.\n"
					"\t-no  Do not save the output.\n"
					"\t-e   Filename extension [=jpg]\n"
					"\t-sq initial_value final_value\n"
					"\t    Tells vid2im to generate a sequence of images\n"
					"\tstarting at filename<initial_value>.ext and\n"
					"\tending in filename<initial_value>.ext\n"
					"\tBy default the sequence range is assumed to be between 0 and 10\n"
					"\twith zero-padding = 3, i.e. 000, 001, ..., 010.\n"
					"\t-zp num   Pad filename suffix with num zeros.\n"
					"\t-fps num  Sets the frame acquisition rate (in frames per second).\n"
					"\t          This option is ignored in the current implementation.\n"
					"\n"
					"Author: Miguel Torres Torriti (c) 2007.12.15, v. 1.\n"
				);
				return -2;
			}
			else if (strstr(argv[i],"-e")!=NULL)
			{
				sprintf((*cmdlineparam).fileextension,"%s",argv[i+1]);
				i++;
			}
			else if (strstr(argv[i],"-sq")!=NULL)
			{
				(*cmdlineparam).iv=atoi(argv[i+1]);
				(*cmdlineparam).fv=atoi(argv[i+2]);
				i=i+2;
				printf("v0:%d, v1:%d\n",(*cmdlineparam).iv,(*cmdlineparam).fv);
			}
			else if (strstr(argv[i],"-zp")!=NULL)
			{
				sprintf((*cmdlineparam).filenameformat,"%%s%%0%dd.%%s",atoi(argv[i+1]));	// Builds file suffix format string "%s%0<n>d"
																							// where <n> is the number for zero padding.
				i++;
			}
			else if (strstr(argv[i],"-fps")!=NULL)
			{
				(*cmdlineparam).fps = atof(argv[i+1]);			
				i++;
			}
			else
			{
				if ((*cmdlineparam).verbose_flag) printf("Invalid option detected: %s\n", argv[i]);
			}
		}
		else if (i==1)
		{
			// Generate the output filename tokenizing the input filename
			sprintf((*cmdlineparam).filename_in, "%s", argv[i]);
			if ((*cmdlineparam).verbose_flag) printf("Input filename: %s\n", (*cmdlineparam).filename_in);
		}
		else
		{
			sprintf((*cmdlineparam).basefilename, "%s", argv[i]);
			if ((*cmdlineparam).verbose_flag) printf("New output base filename: %s\n", (*cmdlineparam).basefilename);
		}
	}

	return 0;
}

int main( int argc, char** argv )
{
	CvCapture* capture = 0;
	IplImage* frame = 0;	       // Frame
	char filename[512];            // Frame filename
	int fnum=0;		               // Frame current number
	struct cmdlinestruct cmdlineparam;	   // Command line parameters structure

	initcmdlinestruct( &cmdlineparam );		

	// Parse the command line parameters 
	if (parsecmdline(argc, argv, &cmdlineparam)==-2)
		return 0;

	// Print welcome message and the OpenCV version
	if (cmdlineparam.verbose_flag)
	{
		printf ("vid2im: Conversion from video to image sequence using OpenCV version %s (%d.%d.%d)\n",
	    CV_VERSION, CV_MAJOR_VERSION, CV_MINOR_VERSION, CV_SUBMINOR_VERSION);

		printf( "Hot keys: \n"
			    "\tESC - quit the program\n"
				"Type vid2im -h for additional help.\n" );
	}
    
    if( argc == 1 || (argc == 2 && strlen(argv[1]) == 1 && isdigit(argv[1][0])))
        capture = cvCaptureFromCAM( argc == 2 ? argv[1][0] - '0' : 0 );
    else if( argc >= 2 )
        capture = cvCaptureFromAVI( argv[1] );

    if( !capture )
    {
        fprintf(stderr,"Could not initialize capturing...\n");
        return -1;
    }

    cvNamedWindow( "vid2im", 0 );
    //cvSetMouseCallback( "vid2im", on_mouse, 0 );

	fnum = cmdlineparam.iv;
    for(;;)
    {
		//IplImage* frame = 0;
		frame = 0; // now frame is created outside the loop, so only the pointer needs to be cleared.

		double t = (double)cvGetTickCount();
		
		frame = cvQueryFrame( capture ); // This function is just a combination of cvGrabFrame and cvRetrieveFrame in one call. 

        if( !frame )
            break;
		
		cvShowImage( "vid2im", frame );

		int c = cvWaitKey(10);
        if( (char)c == 27 )
            break;

		//fnum = fnum > cmdlineparam.fv ? cmdlineparam.iv : fnum;
		if (fnum > cmdlineparam.fv)
			fnum = cmdlineparam.iv;
		sprintf(filename,cmdlineparam.filenameformat,cmdlineparam.basefilename,fnum,cmdlineparam.fileextension);
		cvSaveImage(filename, frame);
		fnum++;

		t = (double)cvGetTickCount() - t;
        printf( "%s | capture time = %.2f ms/frame - fps = %.2f\n",
			filename,
			t/((double)cvGetTickFrequency()*1000.),
			((double)cvGetTickFrequency()*1.e6)/t);
	}

	cvReleaseCapture( &capture );
    cvDestroyWindow("vid2im");
	return 0;
}
